/////////////////////////////////////////////////
//              eGizmo SUMOPROX BOT                //
//                                             //
// This programs a sumobot with built in line  //
// sensor to avoid lines and stay in a ring.   //
// It also has an ultrasonic sensor that       //
// tells the bot to ram the target but avoids  //
// the ring if detected.                       //
//                                             //
// Codes by:                                   //
// e-Gizmo Mechatronix Central                 //
// Taft, Manila, Philippines                   //
// http://www.egizmo.com                       //
// July 6, 2013                                //
/////////////////////////////////////////////////

// Connections:

#define line1 5    // Line sensor
#define line2 6    // Line sensor
#define line3 7    // Line sensor
#define motor1 8   // Motor control HIGH = FWD, LOW = BWD
#define motor2 9   // Speed input
#define motor3 10  // Speed input
#define motor4 11  // Motor control HIGH = FWD, LOW = BWD
#define infrared 12 // Infrared sensor

int speed;
int ramspeed = 250;              // Ram speed
int safespeed = 100;              // Neutral speed
int timer = 10;
int rotate;
int rot_time = 5;

int liner = 0;
int randomizer = 0;

float distance;                  // Distance measured by the ultrasonic sensor

void setup() {                
  Serial.begin(9600);

  pinMode(motor1, OUTPUT);       // Set all motor driver pins to output
  pinMode(motor2, OUTPUT);
  pinMode(motor3, OUTPUT);
  pinMode(motor4, OUTPUT);

  pinMode(line1, INPUT);         // All line sensor pins should be set as input
  pinMode(line2, INPUT);
  pinMode(line3, INPUT);
  pinMode(infrared,INPUT);
}

void loop()  {

  int lineSense1 = digitalRead(line1);
  int lineSense2 = digitalRead(line2);
  int lineSense3 = digitalRead(line3);
  int infraSense = digitalRead(infrared);


  if(infraSense == 0)
  {
    CHARGE();
  }


  if((lineSense1 == 1 || lineSense2 == 1) || lineSense3 == 1)
  {
    liner+=1;
  }

  if(infraSense == 1 && liner == 0)
  {
    safeDrive();
  }

  //////////// If there is an enemy, RAM IT! /////////////
  if(infraSense == 1 && liner == 1)
  {
    ROTATE();
    liner = 0;
  }




}

// PRESET FUNCTIONS:

void safeDrive()
{    
  digitalWrite(motor1, HIGH);
  digitalWrite(motor4, HIGH);

  analogWrite(motor2, safespeed);    
  analogWrite(motor3, safespeed);    
  delay(100);   
  timer--;
}

void WIGGLE()      // One can use this instead of the rotate
{
  digitalWrite(motor1,LOW);
  digitalWrite(motor2,HIGH);
  digitalWrite(motor3,HIGH);
  digitalWrite(motor4,HIGH);
  delay(200);
  digitalWrite(motor1,HIGH);
  digitalWrite(motor2,HIGH);
  digitalWrite(motor3,HIGH);
  digitalWrite(motor4,LOW);
  delay(200);
  digitalWrite(motor1,LOW);
  digitalWrite(motor2,LOW);
  digitalWrite(motor3,LOW);
  digitalWrite(motor4,LOW);

}

void ROTATE()    // Default reaction after timer runs out
{
  digitalWrite(motor1,LOW);
  digitalWrite(motor4,HIGH);
  digitalWrite(motor2,HIGH);
  digitalWrite(motor3,HIGH);

  delay(1000);
}

void RETREAT()
{
  digitalWrite(motor1,LOW);
  digitalWrite(motor4,LOW);

  for(speed = 100; speed>=0; speed--)
  {
    analogWrite(motor2, speed);    
    analogWrite(motor3, 125);    
    delay(10);    
  }  
  timer=10;
}


void STOP()
{
  digitalWrite(motor1,HIGH);
  digitalWrite(motor4,HIGH);

  analogWrite(motor2,1);
  analogWrite(motor3,1);
  delay(10);
}

void CHARGE()
{
  digitalWrite(motor1,HIGH);
  analogWrite(motor2,ramspeed);
  analogWrite(motor3,ramspeed);
  digitalWrite(motor4,HIGH);
}




